<?php

/**
 * Description of class-fscf-display
 * Display class to display the contact form on user website.
 * Functions are called statically, so no need to instantiate the class
 * @authors Mike Challis and Ken Carlson
 */

class FSCF_Display {

	static $global_options, $form_options;
	static $form_content = array();  // Sanitized content submitted by contact form POST of from defaults
	static $form_errors = array();	// error messages from form processing (key = field name)
	static $style = array();		// styles for formatting the form
	static $contact_error;
	static $form_id_num;
	static $contacts;
	static $req_field_ind, $ctf_field_size, $form_action_url, $aria_required;
	static $have_attach = '';
	static $printed_tooltip_filetypes;
	static $add_fscf_script;

	static function process_short_code($atts) {
		// Process shortcode and display the form
		// and decide whether to send the email or not

        self::$add_fscf_script = true;  // condition flag for add styles and scripts to page or post the form is on

		// Extract shortcode atts
		extract( shortcode_atts( array(
					'form'		 => '',
					'redirect'	 => '',
					'hidden'	 => '',
					'email_to'	 => '',
					), $atts ) );

		// Verify form number
		self::$global_options = FSCF_Util::get_global_options();

		self::$form_id_num = '1';
		if ( isset($form) && is_numeric($form) && $form <= self::$global_options['max_form_num'] ) {
		   self::$form_id_num = (int)$form;
		} else {
           echo __( 'Contact Form Shortcode Error: Invalid form number in shortcode.', 'si-contact-form' );
           return;
        }

		$frm_id = self::$form_id_num;

		// Get the form options
		self::$form_options = FSCF_Util::get_form_options( self::$form_id_num, false );  // Don't use defaults if it doesn't exist

		if ( ! self::$form_options ) {
			// Form does not exist in options table
			// Display error message and return
            echo  sprintf( __( 'Contact Form Shortcode Error: Form %s does not exist', 'si-contact-form' ), self::$form_id_num );
            return;
		}

		// Update some language
		// The update_lang function receives the array by reference, so it can be changed
		FSCF_Util::update_lang(self::$form_options);

		// Store shortcode atts
		// http://www.fastsecurecontactform.com/shortcode-options
        if (self::$global_options['enable_php_sessions'] == 'true') { // this feature only works when PHP sessions are enabled
			$_SESSION["fsc_shortcode_redirect_$frm_id"] = $redirect;
			$_SESSION["fsc_shortcode_hidden_$frm_id"] = $hidden;
			$_SESSION["fsc_shortcode_email_to_$frm_id"] = $email_to;
        } else {
           if ( !empty($redirect) || !empty($hidden) || !empty($email_to)) {
            // trying to use shorcode attributes with the required PHP sessions setting turn off
            // Display error message and return
            echo  __( 'Contact Form Shortcode Error: Using shorcode attributes requires the PHP sessions setting to be enabled in form settings.', 'si-contact-form' );
            return;
           }
        }

		self::$form_action_url = self::get_form_action_url();

		// initialize vars
		self::$contact_error = 0;

		// Save parameters from query string, if any
		self::get_query_parms();
		
		self::$req_field_ind = ( self::$form_options['req_field_indicator_enable'] == 'true' ) ? '<span '.self::convert_css( self::$form_options['required_style']).'>'.self::$form_options['req_field_indicator'] . '</span>' : '';

		// See if a form has been processed, and if so, if there were errors
		if ( FSCF_Process::$form_processed && FSCF_Process::$form_id_num == self::$form_id_num  && empty(FSCF_Process::$form_errors) ) {
			// Form was processed and has no errors--display thank you message
			$string = self::display_thank_you();
		} else {
			if ( ! empty(FSCF_Process::$form_errors) && FSCF_Process::$form_id_num == self::$form_id_num ) {
				// The form was processed, but had errors
				if ( ! empty(FSCF_Process::$form_data) ) {
					// If this is not true, there is an internal error...
					self::$form_content = array_merge( self::$form_content, FSCF_Process::$form_data );
					self::$form_errors  = array_merge( self::$form_errors, FSCF_Process::$form_errors );
					// XXX later, improve variable usage for error tracking?
					self::$contact_error = true;
				}
			}
			// ***** Display the Form *****
            $string = "\n<div class='fscf_clear'></div>\n" . self::$form_options['welcome'];
			$string = self::display_form($string);
		}

		return($string);
	}	// end function process_short_code()


  static function get_honeypot_slugs($fields) {
  // filter a list of field names that are not currently used on the form
		$decoy_fields = array( 'address','suite','company','phone','title','city','state','fax','newsletter','webites','zipcode','address2','firstname','lastname','birthday');

		if ($fields && is_array($fields)) {
			foreach ($decoy_fields as $index => $decoy) {
				if (isset($fields[$decoy])) {
					unset($decoy_fields[$index]);
				}
			}
		}

		sort($decoy_fields);
		return $decoy_fields;
	}

	static function get_todays_honeypot_slug($fields) {
       // find a decoy field name that is not currently used on the form
       // change it each day of the week
		$decoy_fields = self::get_honeypot_slugs($fields);
		$max = count($decoy_fields);
         if ($max > 5)
		      $index = date('w');
         else if ($max > 0)
              $index = 0;
         else
              return 'email456';

        return $decoy_fields[$index];
	}


	static function display_form($string) {
		// Build the code to display the form in $string and return it
		// The form code will be appended to $string and returned

		global $captcha_path_cf; // used by secureimage.php
		$captcha_path_cf = FSCF_CAPTCHA_PATH;

		// Set up the styles for the form
		self::$style['notes'] = self::convert_css( self::$form_options['notes_style'] );
		self::$style['form'] = self::convert_css( self::$form_options['form_style'] );
		self::$style['border'] = self::convert_css( self::$form_options['border_style'] );
		self::$style['select'] = self::convert_css( self::$form_options['select_style'] );
		self::$style['title'] = self::convert_css( self::$form_options['title_style'] );
		self::$style['field'] = self::convert_css( self::$form_options['field_style'] );
		self::$style['field_div'] = self::convert_css( self::$form_options['field_div_style'] );
		self::$style['error'] = self::convert_css( self::$form_options['error_style'] );
		self::$style['required'] = self::convert_css( self::$form_options['required_style'] );
		self::$style['submit_div'] = self::convert_css(self::$form_options['submit_div_style']);
		self::$style['submit'] = self::convert_css(self::$form_options['button_style']);
		self::$style['reset'] = self::convert_css(self::$form_options['reset_style']);

		self::$ctf_field_size = absint( self::$form_options['field_size'] );

		self::$aria_required = ' aria-required="true" ';
		
		if ( self::$contact_error )
			self::$style['form'] = str_replace( 'display: none;', '', self::$style['form'] );


		$string .= '
<!-- Fast Secure Contact Form plugin ' . FSCF_VERSION . ' - begin - FastSecureContactForm.com -->
<div id="FSContact' . self::$form_id_num . '" ' . self::$style['form'] . ' class="fscf-container">';

        $form_attributes = '';
        if( !empty(self::$form_options['form_attributes']) )
                $form_attributes = self::$form_options['form_attributes'].' ';

		$string .= '
<form ' . self::$have_attach . 'action="' . esc_url( self::$form_action_url ) . '#FSContact'
				. self::$form_id_num . '" id="si_contact_form' . self::$form_id_num . '" '.$form_attributes.'method="post">
	';

		if ( self::$form_options['border_enable'] == 'true' ) {
			$string .=	'<fieldset ' . self::$style['border'] . '>';
			if ( self::$form_options['title_border'] != '' ) {
				$string .= '<legend>';
				$string .= self::$form_options['title_border'];
				$string .= '</legend>';
			} 
		}
		// check attachment directory
		$frm_id = self::$form_id_num;  // needed for use w/in "" below.. can't use self::
		if ( self::$have_attach ) {
			self::init_temp_dir( FSCF_ATTACH_DIR );
			if ( self::$form_options['php_mailer_enable'] == 'php' ) {
				self::set_form_error("fscf_attach_dir$frm_id", __( 'Attachments are only supported when the Send Email function is set to WordPress. You can find this setting on the contact form settings page.', 'si-contact-form' ) );
			}
			if ( ! is_dir( FSCF_ATTACH_DIR ) ) {
				self::set_form_error("fscf_attach_dir$frm_id", __( 'The temporary folder for the attachment field does not exist.', 'si-contact-form' ) );
			} else if ( ! is_writable( FSCF_ATTACH_DIR ) ) {
				self::set_form_error("fscf_attach_dir$frm_id", __( 'The temporary folder for the attachment field is not writable.', 'si-contact-form' ) );
			} else {
				// delete files over 3 minutes old in the attachment directory
				self::clean_temp_dir( FSCF_ATTACH_DIR, 3 );
			}
		}
		
     	// print input error message
		if ( self::$contact_error ) {
			// There are errors, so print the generic error message
			$string .= '      <div ' . self::$style['error'] . '>' . "\n";
			$string .= (self::$form_options['error_correct'] != '') ? self::$form_options['error_correct'] : __( 'Please make corrections below and try again.', 'si-contact-form' );
			$string .= "\n    </div>\n";
			
			// Print errors that appear at the top of the form
			$string .= self::echo_if_error("fscf_attach_dir$frm_id");
			$string .= self::echo_if_error('akismet');
		}
		
		// Get the email-to list
		self::$contacts = self::get_contact_list(self::$form_id_num, self::$form_options['email_to']);
		
		if ( empty( self::$contacts ) ) {	// was $ctf_contacts
			$string .= '
		<div ' . self::$style['error'] . '>' . __( 'ERROR: Misconfigured email address in options.', 'si-contact-form' )
         . "\n    </div>\n";
		}

		if ( self::$form_options['req_field_label_enable'] == 'true' && self::$form_options['req_field_indicator_enable'] == 'true' ) {
			$string .= '<div ' . self::$style['required'] . '>' . "\n";
			$string .= (self::$form_options['tooltip_required'] != '') ? '<span class="required">' . self::$form_options['req_field_indicator'] . '</span> ' . self::$form_options['tooltip_required'] : '<span class="required">' . self::$form_options['req_field_indicator'] . '</span> ' . __( 'indicates required field', 'si-contact-form' );
			$string .= "\n    </div>\n";
		}

		if ( self::$global_options['enable_php_sessions'] == 'true' ) { // this feature only works when PHP sessions are enabled
			if ( ! empty( $_SESSION["fsc_shortcode_hidden_$frm_id"] ) ) {
				$hidden_fields = self::get_hidden_fields();
				if ( ! empty( $hidden_fields ) ) {
					foreach ( $hidden_fields as $key => $value ) {
						$string .= "\n" . '   <input type="hidden" name="' . esc_attr( $key ) . '" value="' . esc_attr( $value ) . '" />'. "\n";
					}
				}
			} else {
				unset( $_SESSION["fsc_shortcode_hidden_$frm_id"] );
			}
		}

		// Add a hidden field if this is the admin preview, so that we return to the preview after submit
		if ( is_admin() ) {
			$string .= '<input type="hidden" name="ctf_action" value="' . __('Preview Form', 'si-contact-form') . '" />
';
		}

		$string .= '<input type="hidden" id="fsc_submitted_'.self::$form_id_num.'" name="fsc_submitted" value="0" />
';

		// form forgery detection token.
		$string .= '<input type="hidden" name="fs_postonce_'.self::$form_id_num.'" value="'. wp_hash( time() ).','.time() .'" />
';

		$string .= '<input type="hidden" name="si_contact_action" value="send" />
<input type="hidden" name="form_id" value="' . self::$form_id_num . '" />
';

		// If there are multiple mail-to contacts, display a select form
		if ( count( self::$contacts ) > 1 ) {

			$string .= '
        <div ' . self::$style['title'] . '>
                <label for="CID' . self::$form_id_num . '">';
			$string .= (self::$form_options['title_dept'] != '') ? self::$form_options['title_dept'] : __( 'Department to contact:', 'si-contact-form' );
			$string .= self::$req_field_ind . '</label>
        </div>
        <div ' . self::$style['field_div'] . '>
                ' . self::echo_if_error( 'contact' ) . '
                <select ' . self::$style['select'] . ' id="si_contact_CID' . self::$form_id_num . '" name="mailto_id" ' . self::$aria_required . '>
';
			$string .= '      <option value="">';
			$string .= (self::$form_options['title_select'] != '') ? esc_html( self::$form_options['title_select'] ) : esc_html( __( 'Select', 'si-contact-form' ) );
			$string .= '</option>' . "\n";

			if ( ! isset( $cid ) && '' != self::$form_content['mailto_id'] ) {
				$cid = (int) self::$form_content['mailto_id'];
			}
			
			$selected = '';

			foreach ( self::$contacts as $k => $v ) {
				if ( !empty( $cid ) && $cid == $k ) {
					$selected = ' selected="selected"';
				}
				$string .= '                <option value="' . $k . '"' . $selected . '>' . esc_attr($v['CONTACT']) . '</option>' . "\n";
				$selected = '';
			}
			$string .= '                </select>
      </div>' . "\n";
		} else {
			$string .= '<input type="hidden" name="mailto_id" value="1" />
';
		}

		$open_fieldset = false;	// is a fieldset field open?
		// A div class="fscf-clear" is used to group a field with any that follow it
		// $open_div tracks whether this div is currently open
		$open_div = false;
		$date_fields = array();		// List of date fields
		self::$printed_tooltip_filetypes = 0;

		// ********** Go through all the fields and print them **********
		$fields_in_use = array();
		foreach ( self::$form_options['fields'] as $key => $field ) {
			if ( 'true' == $field['disable'] ) continue;
            $fields_in_use[$field['slug']] = 1;
			if ( 'true' == $field['follow'] && $open_div ) {
				$string .= '<div class="fscf-field-follow">';
			} else {
				if ( $open_div ) {
					// close the preceeding div used for grouping
					$string .= "</div>\n";
					$open_div = false;
				}
				if ( 'fieldset' == $field['type'] || 'fieldset-close' == $field['type'] ) {
					$string .= "\n" . '<div class="fscf-clear"></div>';
				} else {
					$open_div = true;
					$string .= "\n" . '<div class="fscf-clear">' . "\n" . '  <div class="fscf-field-left">';
				}
			}

			// Display code common to all/most field types
			if ( ! in_array( $field['type'], array('fieldset', 'fieldset-close', 'hidden') ) ) {
				if ( $field['notes'] != '' ) {
					$string .= self::ctf_notes( $field['notes'] );
				}
				if ( 'checkbox' != $field['type'] && $field['standard'] < 1 ) {
					// Standard field labels can be changed in options, so don't print them here
					// single checkbox labels are printed next to the checkbox
					$string .= "\n   <div ";
					$string .= ( $field['label_css'] != '') ? self::convert_css( $field['label_css'] ) : self::$style['title'];
					$string .= '>
      <label for="si_contact_ex_field' . self::$form_id_num . '_' . $key . '">' . esc_html($field['label']);
					$string .= ( 'true' == $field['req'] ) ? self::$req_field_ind : '';
					$string .= "</label>\n   </div>";
				}
				self::$aria_required = ( $field['req'] && 'true' == self::$form_options['aria_required'] ) ? ' aria-required="true" ' : '';
			}

			switch ( $field['type'] ) {
				case 'fieldset' :
					if ( $open_fieldset )
						$string .= "</fieldset>\n";
					if ( $field['notes'] != '' ) {
						$string .= self::ctf_notes( $field['notes'] );
					}
					$string .= '<fieldset ';
					$string .= ($field['label_css'] != '') ? self::convert_css( $field['label_css'] ) : self::$style['border'];
					$string .= '>
					<legend>' . esc_html($field['label']) . "</legend>\n";
					$open_fieldset = true;
					break;
					
				case 'fieldset-close' :
					if($open_fieldset)
					   $string .=   "</fieldset>\n";
					$open_fieldset = false;					
					break;
					
				case 'hidden' :
					$string .= '      <input type="hidden" id="si_contact_ex_field'.self::$form_id_num.'_'.$key.' name="' . $field['slug'] . '" value="'
						. esc_attr(self::$form_content[$field['slug']]) . '" />' . "\n";					
					break;
				
				case 'password' :
					$string .= '    <div '.self::$style['field_div'].'>'.self::echo_if_error( $field['slug'] ) . "\n"
						. '      <input '.self::$style['field'].' type="password" id="si_contact_ex_field'.self::$form_id_num.'_'.$key.'" name="' . $field['slug'] . '" value=""'
//					No defaults for password!
						. '" '.self::$aria_required.' ';
					if($field['max_len'] != '')
						$string .=  ' maxlength="'.$field['max_len'].'" ';
					if(strpos($field['attributes'],'size')===false)
						$string .= 'size="'.self::$ctf_field_size.'"';
					if($field['attributes'] != '')
					  $string .= ' '.$field['attributes'];
					$string .= " />\n    </div>\n";
					break;
				
				case 'text':
				case 'email':
				case 'url':
					$string .= self::display_field_text($key, $field);
					break;
				
				case 'textarea':
					$string .= self::display_field_textarea($key, $field);
					break;
				
				case 'select' :
				case 'select-multiple':
					$string .= self::display_field_select($key, $field);
					break;

				case 'checkbox':
				case 'checkbox-multiple':
				case 'radio':
					$string .= self::display_field_checkbox($key, $field);
					break;

				case 'date':
					$string .= self::display_field_date($key, $field);
					$date_fields[] = $key;
					break;
				
				case 'time':
					$string .= self::display_field_time($key, $field);
					break;
				
				case 'attachment':
					$string .= self::display_field_attachment($key, $field);
					break;				
			}	// end switch

			if ( 'fieldset' <> $field['type'] && 'fieldset-close' <> $field['type'] && 'hidden' <> $field['type'] ) {
				if ( $field['notes_after'] != '' ) {
					$string .= self::ctf_notes($field['notes_after']);	// was: self::ctf_notes($field['notes_after']);
					}
			}
			if ( $open_div ) {
				$string .= "  </div>\n";	// close field div
			}
			
		} // end foreach (go through fields)

        // action hook for form display after fields
        $string = apply_filters( 'si_contact_display_after_fields', $string, self::$style, self::$form_errors, self::$form_id_num );

		// Are there any date fields?
		if ( count($date_fields) > 0 ) 
			$string .= self::setup_calendar($date_fields);

		
		// ********** Display stuff at tne bottom of form **********
		
		// close final outer container for field and follow groups
		if ( $open_div ) {
			$string .= '</div>
			<div class="fscf-clear"></div>' . "\n";
		}
		
		if ( self::is_captcha_enabled(self::$form_id_num) ) {
			$string .= self::display_captcha() . "\n";
		}

        // hidden empty honeypot field, if enabled
        if ( self::$form_options['honeypot_enable'] == 'true' ) {
           $honeypot_slug = self::get_todays_honeypot_slug($fields_in_use);
      $string .= '        '.self::echo_if_error( $honeypot_slug ).'
      <div style="display:none;">
          <label for="'.$honeypot_slug.self::$form_id_num.'"><small>'.__('Leave this field empty', 'si-contact-form').'</small></label>
          <input type="text" name="'.$honeypot_slug.'" id="'.$honeypot_slug.self::$form_id_num.'" value="" />
        </div>
';
        }

		// Display the submit button
		$string .= "\n<br />\n<div " . self::$style['submit_div'] . '>
		<input type="submit" id="fsc-submit-' . self::$form_id_num . '" ' . self::$style['submit'] . ' class="fsc_submit_button" value="';
		$string .= (self::$form_options['title_submit'] != '') ? esc_attr( self::$form_options['title_submit'] ) : esc_attr( __( 'Submit', 'si-contact-form' ) );
		$string .= '" ';
        if( !empty(self::$form_options['submit_attributes']) )
                $string .= self::$form_options['submit_attributes'].' ';
		if ( self::$form_options['enable_areyousure'] == 'true' ) {
			$msg = (self::$form_options['title_areyousure'] != '') ? esc_html( addslashes( self::$form_options['title_areyousure'] ) ) : esc_html( addslashes( __( 'Are you sure?', 'si-contact-form' ) ) );
			$string .= ' onclick="return confirm(\'' . $msg . '\')" ';

		}
		$string .= '/> ';
		
		if ( self::$form_options['enable_reset'] == 'true' ) {
			$string .= '<input type="reset" id="fsc-reset-' . self::$form_id_num . '" ' . self::$style['reset'] . ' class="fsc_reset_button" value="';
			$string .= (self::$form_options['title_reset'] != '') ? esc_attr( self::$form_options['title_reset'] ) :
			esc_html( __( 'Reset', 'si-contact-form' ) ) . '"';
			$msg = addslashes( __( 'Do you really want to reset the form?', 'si-contact-form' ) );;
			$string .= ' onclick="return confirm(\'' . $msg . '\')" />';
		}
		$string .= "\n</div>\n";
		
		if ( self::$form_options['border_enable'] == 'true' ) {
			$string .= '</fieldset>';
		}

		// Close the form
		$string .= "</form>\n";

		if ( self::$form_options['enable_credit_link'] == 'true' ) {
			$ctf_powered_by_style = self::convert_css( self::$form_options['powered_by_style'] );
			$string .= "\n    <p " . $ctf_powered_by_style . '>' . __( 'Powered by', 'si-contact-form' )
				. ' <a href="http://wordpress.org/extend/plugins/si-contact-form/" target="_blank">' . __( 'Fast Secure Contact Form', 'si-contact-form' ) 
				. "</a></p>\n";
		}
		$string .= '</div>';	// closes fscf-container
		
		$string .= '
<!-- Fast Secure Contact Form plugin '.FSCF_VERSION.' - end - FastSecureContactForm.com -->
';

		return($string);
	}	// end function display_form()
	
	static function display_name($key, $field) {
		// Returns the code to display the name field on the form
		global $current_user, $user_ID;
		// Get defaults

		// Find logged in user's WP name (auto form fill feature):
		// http://codex.wordpress.org/Function_Reference/get_currentuserinfo
		if ( isset(self::$form_content[$field['slug']]) &&
        '' == self::$form_content[$field['slug']] &&
        $user_ID != '' &&
        $current_user->user_login != 'admin' &&
        ! current_user_can( 'manage_options' ) &&
        self::$form_options['auto_fill_enable'] == 'true' ) {
			// user logged in (and not admin rights) (and auto_fill_enable set in options)
			self::$form_content[$field['slug']] = $current_user->user_login;
		}

		$string = '';
		$f_name_string = '
    <div ' . self::$style['title'] . '>
      <label for="si_contact_f_name' . self::$form_id_num . '">';
		$f_name_string .= (self::$form_options['title_fname'] != '') ? self::$form_options['title_fname'] : __( 'First Name:', 'si-contact-form' );
		if ( 'true' == $field['req'] )
			$f_name_string .= self::$req_field_ind;
		$f_name_string .= '</label>
    </div>
    <div ' . self::$style['field_div'] . '>' . self::echo_if_error( 'f_name' ) . '
	  <input ' . self::$style['field'] . ' type="text" id="si_contact_f_name' . self::$form_id_num .
		'" name="f_name" value="' . esc_attr( self::$form_content['f_name'] ) . '" ' . 
		self::$aria_required . ' size="' . self::$ctf_field_size . '" />
    </div>';

		$l_name_string = '
    <div ' . self::$style['title'] . '>
	  <label for="si_contact_l_name' . self::$form_id_num . '">';
		$l_name_string .= (self::$form_options['title_lname'] != '') ? self::$form_options['title_lname'] : __( 'Last Name:', 'si-contact-form' );
		if ( 'true' == $field['req'] )
			$l_name_string .= self::$req_field_ind;
		$l_name_string .= '</label>
    </div>
    <div ' . self::$style['field_div'] . '>' . self::echo_if_error( 'l_name' ) . '
	  <input ' . self::$style['field'] . ' type="text" id="si_contact_l_name' . self::$form_id_num . '" name="l_name" value="' . esc_attr( self::$form_content['l_name'] ) . '" ' . self::$aria_required . ' size="' . self::$ctf_field_size . '" />
    </div>
';

		switch ( self::$form_options['name_format'] ) {
			case 'name':
				$string .= '
    <div ' . self::$style['title'] . '>
      <label for="si_contact_name' . self::$form_id_num . '">';
				$string .= (self::$form_options['title_name'] != '') ? self::$form_options['title_name'] : __( 'Name:', 'si-contact-form' );
				if ( 'true' == $field['req'] )
					$string .= self::$req_field_ind;
				$string .= '</label>
    </div>
    <div ' . self::$style['field_div'] . '>' . self::echo_if_error( 'full_name' ) . '
      <input ' . self::$style['field'] . ' type="text" id="si_contact_name' . self::$form_id_num . '" name="full_name" value="' . esc_attr( self::$form_content[$field['slug']] ) . '" ' . self::$aria_required . ' size="' . self::$ctf_field_size . '" />
    </div>
';
				break;
			case 'first_last':
				$string .= $f_name_string;
				$string .= $l_name_string;
				break;
			case 'first_middle_i_last':
				$string .= $f_name_string;
				$string .= '
    <div ' . self::$style['title'] . '>
      <label for="si_contact_mi_name' . self::$form_id_num . '">';
				$string .= (self::$form_options['title_miname'] != '') ? self::$form_options['title_miname'] : __( 'Middle Initial:', 'si-contact-form' );
				$string .= '</label>
    </div>
    <div ' . self::$style['field_div'] . '>' . self::echo_if_error( 'mi_name' ) . '
       <input ' . self::$style['field'] . ' type="text" id="si_contact_mi_name' . self::$form_id_num . '" name="mi_name" value="' . esc_attr( self::$form_content['mi_name'] ) . '" size="2" />
    </div>';
				$string .= $l_name_string;
				break;
			case 'first_middle_last':
				$string .= $f_name_string;
				$string .= '
    <div ' . self::$style['title'] . '>
      <label for="si_contact_m_name' . self::$form_id_num . '">';
				$string .= (self::$form_options['title_mname'] != '') ? self::$form_options['title_mname'] : __( 'Middle Name:', 'si-contact-form' );
				$string .= '</label>
    </div>
    <div ' . self::$style['field_div'] . '>' . self::echo_if_error( 'm_name' ) . '
      <input ' . self::$style['field'] . ' type="text" id="si_contact_m_name' . self::$form_id_num . '" name="m_name" value="' . esc_attr( self::$form_content['m_name'] ) . '" ' . self::$aria_required . ' size="' . self::$ctf_field_size . '" />
    </div>';
				$string .= $l_name_string;
				break;
		}
		return($string);
	}	// end function display_name()
	
	static function display_email($key, $field) {
		global $current_user, $user_ID;
		$string = '';

        //filter hook for email input type, someone might want to change it from type='text' to type='email'
        $email_input_type = apply_filters( 'si_contact_email_input_type', 'text',  self::$form_id_num);

		// Find logged in user's WP email address (auto form fill feature):
		// http://codex.wordpress.org/Function_Reference/get_currentuserinfo
		if ( '' == self::$form_content[$field['slug']] && $user_ID != '' && $current_user->user_login != 'admin' &&
				!current_user_can( 'manage_options' ) && self::$form_options['auto_fill_enable'] == 'true' ) {
			// user logged in (and not admin rights) (and auto_fill_enable set in options)
			self::$form_content[$field['slug']] = $current_user->user_email;
            if ( 'true' == self::$form_options['double_email'] )
			    self::$form_content['email2'] = $current_user->user_email;
		}

	$string .= "\n    <div " . self::$style['title'] . ">\n"
                . '      <label for="si_contact_email' . self::$form_id_num . '">';
			$string .= (self::$form_options['title_email'] != '') ? self::$form_options['title_email'] : __( 'Email Address:', 'si-contact-form' );
			if ( 'true' == $field['req'] )
				$string .= self::$req_field_ind;
			$string .= "</label>\n    </div>\n";
        $string .= '    <div ' . self::$style['field_div'] . '>' . self::echo_if_error( 'email' )
			. "\n      <input " . self::$style['field'] . ' type="'.$email_input_type.'" id="si_contact_email' . self::$form_id_num
			. '" name="email" value="';
			$string .= esc_attr( self::$form_content[$field['slug']] );
		$string .= '" ' . self::$aria_required . ' size="' . self::$ctf_field_size . '" />'
        . "\n    </div>\n";

		if ( 'true' == self::$form_options['double_email'] ) {
            $string .= "\n    <div " . self::$style['title'] . ">\n"
			. '     <label for="si_contact_email2_' . self::$form_id_num . '">';
			$string .= (self::$form_options['title_email2'] != '') ? self::$form_options['title_email2'] : __( 'Email Address again:', 'si-contact-form' );
			$string .= self::$req_field_ind  
				. "</label>\n    </div>\n    <div " . self::$style['field_div'] . '>' . self::echo_if_error( 'email2' )
                 // XXX later add filter to allow remove the 2nd time message
                . '<span style="font-size:x-small; font-weight:normal;">';
			$string .= (self::$form_options['title_email2_help'] != '') ? self::$form_options['title_email2_help'] : __( 'Please enter your email address a second time.', 'si-contact-form' );
			$string .= "</span><br />\n"
				.'     <input ' . self::$style['field'] . ' type="'.$email_input_type.'" id="si_contact_email2_' . self::$form_id_num
				. '" name="email2" value="' . esc_attr( self::$form_content['email2'] ) . '" ' . self::$aria_required
				. ' size="' . self::$ctf_field_size . '" />'
				. "\n    </div>\n";
		}
		return($string);
	}	// function display_email()

	static function display_field_text($key, $field) {
		// display function for a text, email, or url field type
		$string = '';
		if ( FSCF_NAME_FIELD == $field['standard'] ) return(self::display_name($key, $field));
		if ( FSCF_EMAIL_FIELD == $field['standard'] ) return(self::display_email($key, $field));

		if ( FSCF_SUBJECT_FIELD == $field['standard']) {
			// Display field title for special fields
			$string = '';
			$string .= "\n    <div " . self::$style['title'] . ">\n"
				. '      <label for="si_contact_ex_field'.self::$form_id_num.'_'.$key. '">';
			$string .= (self::$form_options['title_subj'] != '') ? self::$form_options['title_subj'] : __( 'Subject:', 'si-contact-form' );
			if ( 'true' == $field['req'] )
				$string .= self::$req_field_ind;
			$string .= "</label>\n    </div>\n";
		}
        $type = $field['type'];
        if ( 'email' == $field['type'] || 'url' == $field['type']) // XXX fix this later? email or url type was breaking the 'already posted' javascript
           $type = 'text';
        $string .= '    <div ' . self::$style['field_div'] . '>' . self::echo_if_error( $field['slug'] ) ."\n"
		. '      <input ';
		$string .= ($field['input_css'] != '') ? self::convert_css( $field['input_css'] ) : self::$style['field'];
		$string .= ' type="' . $type . '" id="si_contact_ex_field' . self::$form_id_num . '_' . $key . '" name="' . $field['slug'] . '" value="';

        if ( 'url' == $field['type'] ) {
			$string .= esc_url( self::$form_content[$field['slug']] );
		} else {
			$string .= esc_attr( self::$form_content[$field['slug']] );
		}
	
		$string .= '" '. self::$aria_required . ' ';
		if ( $field['max_len'] != '' )
			$string .= ' maxlength="' . $field['max_len'] . '" ';
		if ( strpos( $field['attributes'], 'size' ) === false )
			$string .= 'size="' . self::$ctf_field_size . '"';
		if ( $field['attributes'] != '' )
			$string .= ' ' . $field['attributes'];
		$string .= " />\n    </div>\n";

		return($string);
	}	// end function display_field_text
	
	static function display_field_textarea($key, $field) {
		$string = '';

		if ( FSCF_MESSAGE_FIELD == $field['standard'] ) {
			// Standard field labels can be changed in settings, so print that here
			$string .= "\n    <div " . self::$style['title'] . ">\n"
				. '      <label for="si_contact_ex_field'.self::$form_id_num.'_'.$key. '">';
			$string .= (self::$form_options['title_mess'] != '') ? self::$form_options['title_mess'] : __( 'Message:', 'si-contact-form' );
			if ( 'true' == $field['req'] )
				$string .= self::$req_field_ind;
			$string .= "</label>\n    </div>\n    <div " . self::$style['field_div'] . '>'
				. self::echo_if_error( 'message' ) . "\n      <textarea " . self::$style['field']
				. ' id="si_contact_ex_field'.self::$form_id_num.'_'.$key . '" name="message" ' . self::$aria_required
				. ' cols="' . absint( self::$form_options['text_cols'] ) . '" rows="' . absint( self::$form_options['text_rows'] ) 
				. '">' . esc_textarea( self::$form_content[$field['slug']] ) . "</textarea>\n    </div>\n";
		} else {
			$string	.= "\n    <div " . self::$style['field_div'] . '>' . self::echo_if_error( $field['slug'] ) . "\n"
				.'      <textarea ';
			$string .= ($field['input_css'] != '') ? self::convert_css( $field['input_css'] ) : self::$style['field'];
			$string .= ' id="si_contact_ex_field' . self::$form_id_num . '_' . $key . '" name="' . $field['slug'] . '" ' . self::$aria_required;
			if ( strpos( $field['attributes'], 'cols' ) === false )
				$string .= ' cols="' . absint( self::$form_options['text_cols'] ) . '"';
			if ( strpos( $field['attributes'], 'rows' ) === false )
				$string .= ' rows="' . absint( self::$form_options['text_rows'] ) . '"';

			if ( $field['attributes'] != '' )
				$string .= ' ' . $field['attributes'];
			$string .= '>';
			$string .= (self::$form_options['textarea_html_allow'] == 'true') ? stripslashes( self::$form_content[$field['slug']] ) : esc_textarea( self::$form_content[$field['slug']] );
			$string .= "</textarea>\n    </div>\n";
			}
		return($string);
	}
	
	static function display_field_select($key, $field) {
		$string = '';
		
		// Get the options list
		$opts_array = explode("\n",$field['options']);
		$frm_id = self::$form_id_num;
		if ( '' == $opts_array[0] ) {
			// Error: no options were entered
			self::set_form_error("fscf_select$frm_id", __('Error: No options were entered for a select field in settings.', 'si-contact-form'));
			$string .= self::echo_if_error("fscf_select$frm_id");
		}

		// Display the field
		if ( FSCF_SUBJECT_FIELD == $field['standard']) {
			// Display field title for special fields
			$string = '';
			$string .= "\n    <div " . self::$style['title'] . ">\n"
				. '      <label for="si_contact_ex_field' . self::$form_id_num . '_'.$key .'">';
			$string .= (self::$form_options['title_subj'] != '') ? self::$form_options['title_subj'] : __( 'Subject:', 'si-contact-form' );
			if ( 'true' == $field['req'] )
				$string .= self::$req_field_ind;
			$string .= "</label>\n    </div>";
			// Check for subject_id parm for backward compatibility
			if ( 0 == count(self::$form_content[$field['slug']]) && '' != self::$form_content['subject_id'])
				self::$form_content[$field['slug']][] = self::$form_content['subject_id'];
		}		

		$mult = ( 'select-multiple' == $field['type'] ) ? ' multiple="multiple"' : '';
		$string .= "\n    <div ".self::$style['field_div'].'>'.self::echo_if_error( $field['slug'] )
		."\n      <select ";
		$string .= ($field['input_css'] != '') ? self::convert_css($field['input_css']) : self::$style['field'];
		$string .= ' id="si_contact_ex_field'.self::$form_id_num.'_'.$key.'" name="' . $field['slug'].'[]"';
		if($field['attributes'] != '')
			$string .= ' '.$field['attributes'];
		$string .= $mult .">\n";

		$opts_cnt = 1;
		$selected = '';
		foreach ( $opts_array as $opt ) {
			$opt = trim($opt);
            if ( is_array(self::$form_content[$field['slug']]) ) {
			   if ( count(self::$form_content[$field['slug']]) > 0 ) {
				   if ( in_array( $opts_cnt, self::$form_content[$field['slug']] ) )
					   $selected = ' selected="selected"';
			   }
            }

			if ($opts_cnt == 1 && 'select-multiple' != $field['type'] && preg_match('/^\[(.*)]$/', $opt, $matches)) {// "[Please select]" becomes "Please select"
				$string .= '        <option value=""'.$selected.'>'.esc_attr($matches[1]).'</option>'."\n";
			} else {
				$string .= '        <option value="' . $opts_cnt . '"' . $selected . '>'.esc_attr($opt).'</option>'."\n";
			    $opts_cnt++;
            }
			$selected = '';
		}
		$string .= "      </select>\n    </div>";

		return($string);
	}	// end function display_field_select()
	
	static function display_field_checkbox($key, $field) {
		// Displays checkbox, checkbox-multiple, and radio field types
		$string = '';
	
		// Get the options list
		$opts_array = explode("\n",$field['options']);
		if ( '' == $opts_array[0] ) {
			if ( 'checkbox' == $field['type']) {
				// use the field name as the option name
				$opts_array[0] = $field['label'];
			} else {
				// Error: no options were entered
				self::$contact_error = 1;
				self::$form_errors['fscf_checkbox'] = __('Error: No options were entered for a checkbox-multiple field in settings.', 'si-contact-form');
				$string .= self::echo_if_error('fscf_checkbox');
			}
		}
		if ( 'checkbox' == $field['type'] ) {
			// Single checkbox
			$string .= "\n    <div ";
					$string .= ( $field['label_css'] != '') ? self::convert_css( $field['label_css'] ) : self::$style['title'];
					$string .= '>' . self::echo_if_error( $field['slug'] ) . "\n"
			. '      <input type="checkbox" style="width:13px;" id="si_contact_ex_field' . self::$form_id_num . '_'
			. $key . '" name="' . $field['slug'] . '" value="1" ';
			if ( '1' == self::$form_content[$field['slug']] ) {
				$string .= 'checked="checked"';
			} 

			if ( $field['attributes'] != '' )
				$string .= ' ' . $field['attributes'];
			$string .= ' /> <label style="display:inline;" for="si_contact_ex_field' . self::$form_id_num . '_' . $key . '">'
			. $field['label'];
			$string .= ( 'true' == $field['req'] ) ? self::$req_field_ind : '';
			$string	.= "</label>\n    </div>\n";		
	} else {
		// checkbox-multiple or radio
		if ( 'checkbox-multiple' == $field['type'] ) {
			$ftype = 'checkbox';
		} else {
			$ftype = 'radio';
		}
		$string .= "\n    <div ";
					$string .= ( $field['label_css'] != '') ? self::convert_css( $field['label_css'] ) : self::$style['title'];
					$string .= '>'. self::echo_if_error( $field['slug'] ) . "\n";
		$opt_cnt = 1;
		foreach ( $opts_array as $opt ) {
			$opt = trim( $opt );
			if ( 'false' == $field['inline'] && $opt_cnt > 1 )
				$string .= '    <br />
';
			$string .= '      <span style="white-space:nowrap;">' 
			. '<input type="'. $ftype . '" style="width:13px;" id="si_contact_ex_field'
			. self::$form_id_num . '_' . $key . '_' . $opt_cnt . '" name="' . $field['slug'];
			if ( 'checkbox' == $ftype ) {
				$string .= '[' . $opt_cnt . ']" value="' . $opt_cnt . '" ';
			} else {
				$string .= '" value="' . $opt_cnt . '" ';
			}

			if ( 'checkbox-multiple' == $field['type'] ) {
				if ( count(self::$form_content[$field['slug']]) > 0 
					&& in_array($opt_cnt, self::$form_content[$field['slug']]) ) {
					$string .= ' checked="checked"';
					}
			} else if ( $opt_cnt == self::$form_content[$field['slug']] ) {
				$string .= ' checked="checked"';			
			}

			if ( $field['attributes'] != '' )
				$string .= ' ' . $field['attributes'];
				$string .= ' /> <label style="display:inline;" for="si_contact_ex_field' . self::$form_id_num . '_' . $key . '_'
				. $opt_cnt . '">' . $opt . '</label></span>';
			$opt_cnt++;
		}	// end foreach

		$string .= "\n    </div>\n";

		}  // end else

		return($string);
	}	// end function display_field_checkbox

	static function display_field_date($key, $field) {
		$string = '';

		$cal_date_array = array(
			'mm/dd/yyyy' => esc_html( __( 'mm/dd/yyyy', 'si-contact-form' ) ),
			'dd/mm/yyyy' => esc_html( __( 'dd/mm/yyyy', 'si-contact-form' ) ),
			'mm-dd-yyyy' => esc_html( __( 'mm-dd-yyyy', 'si-contact-form' ) ),
			'dd-mm-yyyy' => esc_html( __( 'dd-mm-yyyy', 'si-contact-form' ) ),
			'mm.dd.yyyy' => esc_html( __( 'mm.dd.yyyy', 'si-contact-form' ) ),
			'dd.mm.yyyy' => esc_html( __( 'dd.mm.yyyy', 'si-contact-form' ) ),
			'yyyy/mm/dd' => esc_html( __( 'yyyy/mm/dd', 'si-contact-form' ) ),
			'yyyy-mm-dd' => esc_html( __( 'yyyy-mm-dd', 'si-contact-form' ) ),
			'yyyy.mm.dd' => esc_html( __( 'yyyy.mm.dd', 'si-contact-form' ) ),
		);

        $string .= "\n    <div " . self::$style['field_div'] . '>' . self::echo_if_error( $field['slug'] ) .
			"\n      <input ";
		$string .= ($field['input_css'] != '') ? self::convert_css( $field['input_css'] ) : self::$style['field'];
		$string .= ' type="text" id="si_contact_ex_field' . self::$form_id_num . '_' . $key . '" name="' . $field['slug'] . '" value="';
		$string .= ( isset( self::$form_content[$field['slug']] ) && self::$form_content[$field['slug']] != '') ? esc_attr( self::$form_content[$field['slug']] ) : $cal_date_array[self::$form_options['date_format']];
		$string .= '" ' . self::$aria_required . ' size="15" ';
		if ( $field['attributes'] != '' )
			$string .= ' ' . $field['attributes'];
		$string .= " />\n    </div>\n";

		return($string);
	}	// end function display_field_date
	
	static function display_field_attachment($key, $field) {
		$string = '';
		
		if ( self::$form_options['php_mailer_enable'] != 'php' ) {
			$string .= "\n    <div " . self::$style['field_div'] . '>' . self::echo_if_error( $field['slug'] )
				. "\n      <input " . self::$style['field'] . ' type="file" id="si_contact_ex_field'
				. self::$form_id_num . '_' . $key . '" name="' . $field['slug'] . '" value="' 
				. esc_attr( self::$form_content[$field['slug']] ) . '" ' . self::$aria_required . ' size="20" ';
			if ( $field['attributes'] != '' )
				$string .= ' ' . $field['attributes'];
			$string .= ' />';
			// This was designed to print the tip if it was not printed on the previous field
			if ( ! self::$printed_tooltip_filetypes) {
				$file_type_message = '<br /><span style="font-size:x-small;">';
				$file_type_message .= (self::$form_options['tooltip_filetypes'] != '') ? self::$form_options['tooltip_filetypes'] : __( 'Acceptable file types:', 'si-contact-form' );
				$file_type_message .= ' ' . self::$form_options['attach_types'] . '.<br />';
				$file_type_message .= (self::$form_options['tooltip_filesize'] != '') ? self::$form_options['tooltip_filesize'] : __( 'Maximum file size:', 'si-contact-form' );
				$file_type_message .= ' ' . self::$form_options['attach_size'] . '.</span>';
                //filter hook for file attachment acceptable types message
                $file_type_message = apply_filters( 'si_contact_file_type_message', $file_type_message, self::$form_options,  self::$form_id_num);
                $string .= $file_type_message;
			}
			self::$printed_tooltip_filetypes++;
			$string .= "    </div>\n";
		}		
		
		return($string);
	}
	
	static function setup_calendar($date_fields) {
		// Set up the popup calendar display for date fields
		$string = '';
		$temp = plugins_url( 'date/ctf_epoch_styles.css?' . time(), FSCF_FILE );
		$string .=
		'<script type="text/javascript">
			var ctf_css = document.createElement(\'link\');
			ctf_css.rel = \'stylesheet\';
			ctf_css.type = \'text/css\';
			ctf_css.href = \'' . plugins_url( 'date/ctf_epoch_styles.css?' . time(), FSCF_FILE ) . '\';
			document.getElementsByTagName(\'head\')[0].appendChild(ctf_css);
			var ctf_daylist = new Array( \'' . __( 'Su', 'si-contact-form' ) . '\',\'' . __( 'Mo', 'si-contact-form' ) . '\',\'' . __( 'Tu', 'si-contact-form' ) . '\',\'' . __( 'We', 'si-contact-form' ) . '\',\'' . __( 'Th', 'si-contact-form' ) . '\',\'' . __( 'Fr', 'si-contact-form' ) . '\',\'' . __( 'Sa', 'si-contact-form' ) . '\',\'' . __( 'Su', 'si-contact-form' ) . '\',\'' . __( 'Mo', 'si-contact-form' ) . '\',\'' . __( 'Tu', 'si-contact-form' ) . '\',\'' . __( 'We', 'si-contact-form' ) . '\',\'' . __( 'Th', 'si-contact-form' ) . '\',\'' . __( 'Fr', 'si-contact-form' ) . '\',\'' . __( 'Sa', 'si-contact-form' ) . '\' );
			var ctf_months_sh = new Array( \'' . __( 'Jan', 'si-contact-form' ) . '\',\'' . __( 'Feb', 'si-contact-form' ) . '\',\'' . __( 'Mar', 'si-contact-form' ) . '\',\'' . __( 'Apr', 'si-contact-form' ) . '\',\'' . __( 'May', 'si-contact-form' ) . '\',\'' . __( 'Jun', 'si-contact-form' ) . '\',\'' . __( 'Jul', 'si-contact-form' ) . '\',\'' . __( 'Aug', 'si-contact-form' ) . '\',\'' . __( 'Sep', 'si-contact-form' ) . '\',\'' . __( 'Oct', 'si-contact-form' ) . '\',\'' . __( 'Nov', 'si-contact-form' ) . '\',\'' . __( 'Dec', 'si-contact-form' ) . '\' );
			var ctf_monthup_title = \'' . __( 'Go to the next month', 'si-contact-form' ) . '\';
			var ctf_monthdn_title = \'' . __( 'Go to the previous month', 'si-contact-form' ) . '\';
			var ctf_clearbtn_caption = \'' . __( 'Clear', 'si-contact-form' ) . '\';
			var ctf_clearbtn_title = \'' . __( 'Clears any dates selected on the calendar', 'si-contact-form' ) . '\';
			var ctf_maxrange_caption = \'' . __( 'This is the maximum range', 'si-contact-form' ) . '\';
			var ctf_cal_start_day = ' . self::$form_options['cal_start_day'] . ';
			var ctf_date_format = \'';

		if ( self::$form_options['date_format'] == 'mm/dd/yyyy' )
			$string .= 'm/d/Y';
		if ( self::$form_options['date_format'] == 'dd/mm/yyyy' )
			$string .= 'd/m/Y';
		if ( self::$form_options['date_format'] == 'mm-dd-yyyy' )
			$string .= 'm-d-Y';
		if ( self::$form_options['date_format'] == 'dd-mm-yyyy' )
			$string .= 'd-m-Y';
		if ( self::$form_options['date_format'] == 'mm.dd.yyyy' )
			$string .= 'm.d.Y';
		if ( self::$form_options['date_format'] == 'dd.mm.yyyy' )
			$string .= 'd.m.Y';
		if ( self::$form_options['date_format'] == 'yyyy/mm/dd' )
			$string .= 'Y/m/d';
		if ( self::$form_options['date_format'] == 'yyyy-mm-dd' )
			$string .= 'Y-m-d';
		if ( self::$form_options['date_format'] == 'yyyy.mm.dd' )
			$string .= 'Y.m.d';

		$string .= '\';
		</script>
		<script type="text/javascript" src="'.plugins_url('date/ctf_epoch_classes.js?'.time(), FSCF_FILE ).'"></script>
		<script type="text/javascript">
		var ';
		$date_var_string = '';
		foreach ( $date_fields as $v ) {
			$date_var_string .= 'dp_cal' . self::$form_id_num . '_' . "$v,";
		}
		$date_var_string = substr( $date_var_string, 0, -1 );
		$string .= "$date_var_string;\n";
        $use_window_onload = true;
        //filter hook to suppress window.onload = function(){} for sites where it can only happen once per page.
        $use_window_onload = apply_filters( 'si_contact_use_window_onload', $use_window_onload,  self::$form_id_num);
        if ($use_window_onload)
		    $string .= 'window.onload = function () {
';
		foreach ( $date_fields as $v ) {
			$string .= 'dp_cal'. self::$form_id_num . '_' . "$v  = new Epoch('epoch_popup" . self::$form_id_num . '_' . "$v','popup',document.getElementById('si_contact_ex_field" . self::$form_id_num . '_' . "$v'));\n";
		}
        if ($use_window_onload)
		   $string .= "};\n";

        $string .= "</script>\n";

		return($string);
	}	// end function setup_calendar()

	static function display_field_time($key, $field) {
		$string = '';
		
		// the time drop down list array will be made automatically by this code
        $string .= "\n    <div " . self::$style['field_div'] . '>' . self::echo_if_error( $field['slug'] )
			. "      <select ";
		$string .= ($field['input_css'] != '') ? self::convert_css( $field['input_css'] ) : self::$style['field'];
		$string .= ' id="si_contact_ex_field' . self::$form_id_num . '_' . $key . '" name="' . $field['slug']
				. '[h]">' . "\n";

		$selected = '';
		// hours
		$tf_hours = (self::$form_options['time_format'] == '24') ? '23' : '12';
		for ( $keyi = (self::$form_options['time_format'] == '24') ? 0 : 1; $keyi <= $tf_hours; $keyi++ ) {
			$keyi = sprintf( "%02d", $keyi );
			if ( self::$form_content[$field['slug']]['h'] != '' ) {
				if ( self::$form_content[$field['slug']]['h'] == "$keyi" ) {
					$selected = ' selected="selected"';
				}
			}
			$string .= '        <option value="' . esc_attr( $keyi ) . '"' . $selected . '>' . esc_html( $keyi ) . '</option>' . "\n";
			$selected = '';
		}
		$string .= '      </select>:<select ';
		$string .= ($field['input_css'] != '') ? self::convert_css( $field['input_css'] ) : self::$style['field'];
		$string .= ' id="si_contact_ex_field' . self::$form_id_num . '_' . $key . 'm" name="' . $field['slug'] . '[m]">' . "\n";
		$selected = '';
		// minutes
		for ( $keyi = 00; $keyi <= 59; $keyi++ ) {
			$keyi = sprintf( "%02d", $keyi );
			if ( self::$form_content[$field['slug']]['m'] != '' ) {
				if ( self::$form_content[$field['slug']]['m'] == "$keyi" ) {
					$selected = ' selected="selected"';
				}
			}
			$string .= '        <option value="' . esc_attr( $keyi ) . '"' . $selected . '>' . esc_html( $keyi ) . '</option>' . "\n";
			$selected = '';
		}
		$string .= '      </select>';
		if ( self::$form_options['time_format'] == '12' ) {
			$string .= '<select ';
			$string .= ($field['input_css'] != '') ? self::convert_css( $field['input_css'] ) : self::$style['field'];
			$string .= ' id="si_contact_ex_field' . self::$form_id_num . '_' . $key . 'ap" name="' . $field['slug'] . '[ap]">' . "\n";
			$selected = '';
			// am/pm
			foreach ( array( esc_html( __( 'AM', 'si-contact-form' ) ), esc_html( __( 'PM', 'si-contact-form' ) ) ) as $k ) {
				if ( self::$form_content[$field['slug']]['ap'] != '' ) {
					if ( self::$form_content[$field['slug']]['ap'] == "$k" ) {
						$selected = ' selected="selected"';
					}
				}
				$string .= '        <option value="' . esc_attr( $k ) . '"' . $selected . '>' . esc_html( $k ) . '</option>' . "\n";
				$selected = '';
			}
			$string .= '      </select>';
		}
		$string .= "\n    </div>\n";

		return($string);
	}	// end function display_field_time()
	
	
	static function get_contact_list($frm_id, $email_to) {
		// Returns a list of email contacts for display
		// $email_to = email to list from form settings
		
		if ( ! self::$global_options )
			self::$global_options = FSCF_Util::get_global_options ( );
		
		$contacts = array();
		$contacts[] = '';	// dummy entry to take up key 0

		// Check for a shortcode mail-to value.  Allowed shortcode email_to: 
		//    Webmaster,user1@example.com (must have name,email)
		// multiple emails allowed
		//    Webmaster,user1@example.com;user2@example.com

		if (self::$global_options['enable_php_sessions'] == 'true') { // this feature only works when PHP sessions are enabled		
		  if ( ! empty($_SESSION["fsc_shortcode_email_to_$frm_id"]) &&
               preg_match( "/,/", $_SESSION["fsc_shortcode_email_to_$frm_id"]  ) ) {
				list($key, $value) = preg_split( '#(?<!\\\)\,#', $_SESSION["fsc_shortcode_email_to_$frm_id"] ); //string will be split by "," but "\," will be ignored
				$key = trim( str_replace( '\,', ',', $key ) ); // "\," changes to ","
				$value = trim( str_replace( ';', ',', $value ) ); // ";" changes to ","
				if ( $key != '' && $value != '' ) {
					$contacts[] = array( 'CONTACT' => FSCF_Util::clean_input($key), 
										 'EMAIL'   => FSCF_Util::clean_input($value) );
				}
		  } else {
             unset($_SESSION["fsc_shortcode_email_to_$frm_id"]);
          }
        }				
		
		if ( count($contacts) == 1 ) {
			// Nothing from shortcode, so generate the mail-to list from the settings.
			// The drop down list array will be made automatically by this code
			// checks for properly configured Email To: addresses in options.

			$contacts_test = trim( $email_to );
			if ( ! preg_match( "/,/", $contacts_test ) ) {
				if ( FSCF_Util::validate_email( $contacts_test ) ) {
					// user1@example.com
					$contacts[] = array( 'CONTACT'	 => __( 'Webmaster', 'si-contact-form' ), 'EMAIL'		 => $contacts_test );
				}
			} else {
				$ctf_ct_arr = explode( "\n", $contacts_test );
				if ( is_array( $ctf_ct_arr ) ) {
					foreach ( $ctf_ct_arr as $line ) {
						// echo '|'.$line.'|' ;
						list($key, $value) = preg_split( '#(?<!\\\)\,#', $line ); //string will be split by "," but "\," will be ignored
						$key = trim( str_replace( '\,', ',', $key ) ); // "\," changes to ","
						$value = trim( $value );
						if ( $key != '' && $value != '' ) {
							if ( !preg_match( "/;/", $value ) ) {
								// just one email here
								// Webmaster,user1@example.com
								$value = str_replace( '[cc]', '', $value );
								$value = str_replace( '[bcc]', '', $value );
								if ( FSCF_Util::validate_email( $value ) ) {
									$contacts[] = array( 'CONTACT'	 => esc_html( $key ), 'EMAIL'		 => $value );
								}
							} else {
								// multiple emails here
								// Webmaster,user1@example.com;user2@example.com;user3@example.com;[cc]user4@example.com;[bcc]user5@example.com
								$multi_cc_arr = explode( ";", $value );
								$multi_cc_string = '';
								foreach ( $multi_cc_arr as $multi_cc ) {
									$multi_cc_t = str_replace( '[cc]', '', $multi_cc );
									$multi_cc_t = str_replace( '[bcc]', '', $multi_cc_t );
									if ( FSCF_Util::validate_email( $multi_cc_t ) ) {
										$multi_cc_string .= "$multi_cc,";
									}
								}
								if ( $multi_cc_string != '' ) { // multi cc emails
									$contacts[] = array( 'CONTACT'	 => esc_html( $key ), 'EMAIL'		 => rtrim( $multi_cc_string, ',' ) );
								}
							}
						}
					} // end foreach
				} // end if (is_array($ctf_ct_arr) ) {
			} // end else
		} // end outer if
		
		unset($contacts[0]); // remove dummy entry.. the array keys now start with 1
		return($contacts);
	}	// end function get_contact_list()
	
	static function convert_css( $string ) {
    if ( preg_match( "/^style=\"(.*)\"$/i", $string, $matches ) ) {
			return 'style="' . esc_attr( $matches[1] ) . '"';
		}
		if ( preg_match( "/^class=\"(.*)\"$/i", $string, $matches ) ) {
			return 'class="' . esc_attr( $matches[1] ) . '"';
		}
		return 'style="' . esc_attr( $string ) . '"';
	}	// end function convert_css
		
	static function get_var($form_id_num,$name) {
		// Gets field value, if any, from query parm
		$value = (isset( $_GET["$form_id_num$name"])) ? FSCF_Util::clean_input($_GET["$form_id_num$name"]) : '';
		return $value;
	}
	
	static function get_query_parms() {
		// Check for query arguements, and store them in self::$form_content
		// If nothing is set by query, put in the default, if one exists
		global $fscf_special_slugs;		// List of reserve slug names

		// initialize vars
		self::$have_attach = '';
		
		// Get any field values from query parms
		// Get special fields
//		$special_slugs = array( 'f_name', 'm_name', 'mi_name', 'l_name', 'email2', 'mailto_id', 'subject_id' );
		foreach ( $fscf_special_slugs as $fld_name ) {
			self::$form_content[$fld_name] = self::get_var(self::$form_id_num,$fld_name);
		}
		
		// XXX Might need to check for English standard field names, e.g. 'name', as well as the actual field name,
		// which might be translated.  If so, set the field name element to the entry for uame, and unset the name element
		
		// Get regular fields
		foreach (self::$form_options['fields'] as $key => $field ) {
			if ( 'true' == $field['disable'] ) continue;
			$fld_name = $field['slug'];
			switch ( $field['type'] ) {
				case 'time' :
					$vars = array();
					$vars['h']  = self::get_var(self::$form_id_num,$fld_name.'_h');
					$vars['m']  = self::get_var(self::$form_id_num,$fld_name.'_m');
					$vars['ap'] = self::get_var(self::$form_id_num,$fld_name.'_ap');
					self::$form_content[$fld_name] = $vars;
					// XXX need to add use of default for time as "xx:xx am/pm"
					break;

				case 'select' :
				case 'select-multiple' :
				case 'checkbox-multiple' :
					// Checkbox and radio are handled in the default case below
					$opts_array = explode("\n",$field['options']);
					$selected = array();
					if ( 'select-multiple' == $field['type'] || 'checkbox-multiple' == $field['type'] ) {
						$opt_cnt = 1;
						foreach ( $opts_array as $opt ) {
							$sel = self::get_var( self::$form_id_num, $fld_name . '_' . $opt_cnt );
							if ( '1' == $sel ) $selected[] = $opt_cnt;
							$opt_cnt++;
						}
					}

					if ( 0 == count($selected) ) {
						// see if a single option was specified
						$ind = self::get_var(self::$form_id_num,$fld_name);
						if ( is_numeric($ind) && isset($opts_array[+$ind-1]) ) {
							$selected[] = $ind;
						} else if ( '' != $field['default'] ) {
							// Get the value from the default setting
							if ( ! false == strpos( $field['default'], ',') ) {
								// Parse a comma delimited option list
								$olist = explode(',', $field['default']);
								foreach ( $olist as $opt ) {
									if ( is_numeric($opt) && isset($opts_array[+$opt-1]) ) $selected[] = $opt;
								}
							} else if ( is_numeric($field['default']) && isset($opts_array[+$field['default']-1]) )
								$selected[] = $field['default'];
						}
					}
					self::$form_content[$fld_name] = $selected;
					break;

				case 'attachment' :
					self::$have_attach = 'enctype="multipart/form-data" '; // for <form post
					self::$form_content[$fld_name] = self::get_var(self::$form_id_num, $fld_name);
					// There is no default value for an attachment field
					break;

				case 'fieldset' :
				case 'fieldset-close' :
				case 'password' :
					break;

				case 'date' :
					// check to be sure that query or default date is a valid date, or do not use it
					$new_date = self::get_var( self::$form_id_num, $fld_name );
					if ( '' != $new_date & FSCF_Process::validate_date( $new_date, self::$form_id_num ) )
						self::$form_content[$fld_name] = $new_date;
					// XXX Consider adding an error or warning message if date field query parm is invalid
					if ( empty( self::$form_content[$fld_name] ) && '' != $field['default'] && FSCF_Process::validate_date( $field['default'], self::$form_id_num ) ) {
						self::$form_content[$fld_name] = $field['default'];
					}
					break;

				default :
					// Special case: the 'full_name' field has 'name' as the query name
					if ( 'full_name' == $fld_name )
						self::$form_content[$fld_name] = self::get_var(self::$form_id_num, 'name');
					else
						self::$form_content[$fld_name] = self::get_var(self::$form_id_num, $fld_name);
					if ( '' == self::$form_content[$fld_name] && '' != $field['default'] ) {
                        if('message' == $fld_name || 'textarea' == $field['type'])
                            self::$form_content[$fld_name] = str_replace('\n', "\n", $field['default']);
                        else
                            self::$form_content[$fld_name] = $field['default'];
                    }

			}	// end switch

		}	// end foreach
		return;
	}	// end function get_query_parms()

	static function get_hidden_fields() {
		// Gets hidden fields from shortcode
		// This is called only if PHP Sessions are enabled
		// Returns an array of hidden fields
		$frm_id = self::$form_id_num;
		$hidden_fields = array();
		$hidden_fields_test = explode( ",", $_SESSION["fsc_shortcode_hidden_$frm_id"]);
		if ( ! empty( $hidden_fields_test ) ) {
			foreach ( $hidden_fields_test as $line ) {
				if ( preg_match( "/=/", $line ) ) {
					list($key, $value) = explode( "=", $line );
					$key = trim( $key );
					$value = trim( $value );
					if ( $key != '' && $value != '' ) 
						$hidden_fields[$key] = $value;
				}
			}
		}
		
		// Save the hidden fields in session for use in processing
		if ( ! empty( $hidden_fields ) )
			$_SESSION["fsc_shortcode_hidden_$frm_id"] = $hidden_fields;
		else
			unset($_SESSION["fsc_shortcode_hidden_$frm_id"]);

		return($hidden_fields);
	}  // end function get_hidden_fields()
	
	static function echo_if_error( $this_error ) {
		// shows contact form errors
		// Settings errors begin with fscf . $form_num (always shown)
		// Entry errors will only be in self::$form_errors for the submitted form
		if ( self::$contact_error ) {
			if ( isset( self::$form_errors[$this_error] ) ) {
				return '
         <div ' . self::$style['error'] . '>' . self::$form_errors[$this_error] . '</div>' . "\n";
			}
		}
	} // end function echo_if_error

	// needed for making temp directories for attachments and captcha session files
	static function init_temp_dir( $dir ) {
		$dir = trailingslashit( $dir );
		// make the temp directory
		wp_mkdir_p( $dir );
		//@chmod( $dir, 0733 );
		$htaccess_file = $dir . '.htaccess';
		if ( !file_exists( $htaccess_file ) ) {
			if ( $handle = @fopen( $htaccess_file, 'w' ) ) {
				fwrite( $handle, "Deny from all\n" );
				fclose( $handle );
			}
		}
		$php_file = $dir . 'index.php';
		if ( !file_exists( $php_file ) ) {
			if ( $handle = @fopen( $php_file, 'w' ) ) {
				fwrite( $handle, '<?php //do not delete ?>' );
				fclose( $handle );
			}
		}
	}	// end function init_temp_dir

	static function clean_temp_dir( $dir, $minutes = 30 ) {
		// needed for emptying temp directories for attachments and captcha session files
		// deletes all files over xx minutes old in a temp directory
		if ( !is_dir( $dir ) || !is_readable( $dir ) || !is_writable( $dir ) )
			return false;

		$count = 0;
		$list = array( );
		if ( $handle = @opendir( $dir ) ) {
			while ( false !== ( $file = readdir( $handle ) ) ) {
				if ( $file == '.' || $file == '..' || $file == '.htaccess' || $file == 'index.php' )
					continue;

				$stat = @stat( $dir . $file );
				if ( ( $stat['mtime'] + $minutes * 60 ) < time() ) {
					@unlink( $dir . $file );
					$count += 1;
				} else {
					$list[$stat['mtime']] = $file;
				}
			}
			closedir( $handle );
			// purge xx amount of files based on age to limit a DOS flood attempt. Oldest ones first, limit 500
			if ( isset( $list ) && count( $list ) > 499 ) {
				ksort( $list );
				$ct = 1;
				foreach ( $list as $k => $v ) {
					if ( $ct > 499 )
						@unlink( $dir . $v );
					$ct += 1;
				}
			}
		}
		return $count;
	}
	
	static function is_captcha_enabled($frm_num) {
		// See if captcha is enabled for this user and this form
		// Returns true or false
		
		if ( ! self::$form_options )
			self::$form_options = FSCF_Util::get_form_options ( $frm_num, $use_defaults=true );

		$captcha_enabled = true;
		if ( self::$form_options['captcha_enable'] !== 'true' ) {
			$captcha_enabled = false; // captcha setting is disabled for si contact
		}
		// skip the captcha if user is loggged in and the settings allow
		if ( is_user_logged_in() && self::$form_options['captcha_perm'] == 'true' ) {
			// skip the CAPTCHA display if the minimum capability is met
			if ( current_user_can( self::$form_options['captcha_perm_level'] ) ) {
				$captcha_enabled = false;
			}
		}
		return ($captcha_enabled);
	}  // end function is_captcha_enabled()
	
	static function display_captcha() {
		// this function adds the captcha to the contact form

		$captchaRequiresError = '';

		$enable_php_sessions = 0;
		if ( self::$global_options['enable_php_sessions'] == 'true' )
			$enable_php_sessions = 1;

		$string = '';

		// Test for some required things, print error message right here if not OK.
		// Code moved in from function captchaCheckRequires() -- only called once
		$captcha_ok = true;
		// Test for some required things, print error message if not OK.
		if ( !extension_loaded( 'gd' ) || !function_exists( 'gd_info' ) ) {
			$captchaRequiresError .= '<p ' . self::$style['error'] . '>' . __( 'ERROR: si-contact-form.php plugin says GD image support not detected in PHP!', 'si-contact-form' ) . '</p>';
			$captchaRequiresError .= '<p>' . __( 'Contact your web host and ask them why GD image support is not enabled for PHP.', 'si-contact-form' ) . '</p>';
			$captcha_ok = false;
		}
		if ( !function_exists( 'imagepng' ) ) {
			$captchaRequiresError .= '<p ' . self::$style['error'] . '>' . __( 'ERROR: si-contact-form.php plugin says imagepng function not detected in PHP!', 'si-contact-form' ) . '</p>';
			$captchaRequiresError .= '<p>' . __( 'Contact your web host and ask them why imagepng function is not enabled for PHP.', 'si-contact-form' ) . '</p>';
			$captcha_ok = false;
		}
		if ( !@strtolower( ini_get( 'safe_mode' ) ) == 'on' && !file_exists( FSCF_CAPTCHA_PATH . '/securimage.php' ) ) {
			$captchaRequiresError .= '<p ' . self::$style['error'] . '>' . __( 'ERROR: si-contact-form.php plugin says captcha_library not found.', 'si-contact-form' ) . '</p>';
			$captcha_ok = false;
		}

		if ( $captcha_ok ) {

			self::$form_options['captcha_image_style'] = 'border-style:none; margin:0; padding:0px; padding-right:5px; float:left;';
			self::$form_options['reload_image_style'] = 'border-style:none; margin:0; padding:0px; vertical-align:bottom;';

			// the captch html
			$string = "\n<div " . self::$style['title'] . ">\n</div>" . "\n<div ";
			self::$style['captcha_div_sm'] = self::convert_css( self::$form_options['captcha_div_style_sm'] );
			self::$style['captcha_div_m'] = self::convert_css( self::$form_options['captcha_div_style_m'] );

			// url for captcha image
			$captcha_url_cf = FSCF_Util::get_captcha_url_cf();
			$securimage_show_url = $captcha_url_cf . '/securimage_show.php?';

			$securimage_size = 'width="175" height="60"';
			if ( self::$form_options['captcha_small'] == 'true' ) {
				$securimage_show_url .= 'ctf_sm_captcha=1&amp;';
				$securimage_size = 'width="132" height="45"';
			}

			$parseUrl = parse_url( $captcha_url_cf );
			$securimage_url = $parseUrl['path'];

			if ( !$enable_php_sessions ) { // no sessions
				self::init_temp_dir( FSCF_CAPTCHA_PATH . '/cache/' );

				// clean out old captcha cache files
				self::clean_temp_dir( FSCF_CAPTCHA_PATH . '/cache/' );
				// pick new prefix token
				$prefix_length = 16;
				$prefix_characters = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz';
				$prefix = '';
				$prefix_count = strlen( $prefix_characters );
				while ( $prefix_length-- ) {
					$prefix .= $prefix_characters[mt_rand( 0, $prefix_count - 1 )];
				}
				$securimage_show_rf_url = $securimage_show_url . 'prefix=';
				$securimage_show_url .= 'prefix=' . $prefix;
			} else {  // no session
				$securimage_show_rf_url = $securimage_show_url . 'ctf_form_num=' . self::$form_id_num;
				$securimage_show_url .= 'ctf_form_num=' . self::$form_id_num;
			}

			$string .= (self::$form_options['captcha_small'] == 'true') ? self::$style['captcha_div_sm'] : self::$style['captcha_div_m'];
			$string .= ">\n" . '      <img class="ctf-captcha" id="si_image_ctf' . self::$form_id_num . '" ';
			$string .= (self::$form_options['captcha_image_style'] != '') ? 'style="' . esc_html( self::$form_options['captcha_image_style'] ) . '"' : '';
			$string .= ' src="' . $securimage_show_url . '" ' . $securimage_size . ' alt="';
			$string .= (self::$form_options['tooltip_captcha'] != '') ? esc_html( self::$form_options['tooltip_captcha'] ) : esc_html( __( 'CAPTCHA Image', 'si-contact-form' ) );
			$string .='" title="';
			$string .= (self::$form_options['tooltip_captcha'] != '') ? esc_html( self::$form_options['tooltip_captcha'] ) : esc_html( __( 'CAPTCHA Image', 'si-contact-form' ) );
			$string .= '" />' . "\n";

			if ( !$enable_php_sessions ) { // no sessions
				$string .= '      <input id="captcha_prefix_' . self::$form_id_num . '" type="hidden" name="captcha_prefix_' . self::$form_id_num . '" value="' . esc_attr( $prefix ) . '" />' . "\n";
			}

			$string .= '     <div id="si_refresh_ctf' . self::$form_id_num . '">' . "\n";
			$string .= '        <a href="#" rel="nofollow" title="';
			$string .= (self::$form_options['tooltip_refresh'] != '') ? esc_html( self::$form_options['tooltip_refresh'] ) : esc_html( __( 'Refresh Image', 'si-contact-form' ) );
			if ( !$enable_php_sessions ) { // no sessions
				$string .= '" onclick="si_contact_captcha_refresh(\'' . self::$form_id_num . '\',\'' . $securimage_url . '\',\'' . $securimage_show_rf_url . '\'); return false;">' . "\n";
			} else {
				$string .= '" onclick="document.getElementById(\'si_image_ctf' . self::$form_id_num . '\').src = \'' . $securimage_show_url . '&amp;sid=\'' . ' + Math.random(); return false;">' . "\n";
			}
			$string .= '        <img src="' . $captcha_url_cf . '/images/refresh.png" width="22" height="20" alt="';
			$string .= (self::$form_options['tooltip_refresh'] != '') ? esc_html( self::$form_options['tooltip_refresh'] ) : esc_html( __( 'Refresh Image', 'si-contact-form' ) );
			$string .= '" ';
			$string .= (self::$form_options['reload_image_style'] != '') ? 'style="' . esc_html( self::$form_options['reload_image_style'] ) . '"' : '';
			$string .= ' onclick="this.blur();" /></a>
     </div>
</div>

<div ' . self::$style['title'] . '>
     <label for="si_contact_captcha_code' . self::$form_id_num . '">';
			$string .= (self::$form_options['title_capt'] != '') ? self::$form_options['title_capt'] : __( 'CAPTCHA Code:', 'si-contact-form' );
			$string .= self::$req_field_ind . '</label>
</div>
<div ' . self::convert_css( self::$form_options['field_div_style'] ) . '>'
					. self::echo_if_error( 'captcha' )
					. "\n     <input " . self::convert_css( self::$form_options['captcha_input_style'] )
					. ' type="text" value="" id="si_contact_captcha_code' . self::$form_id_num . '" name="captcha_code" ' . self::$aria_required . ' size="' . absint( self::$form_options['captcha_field_size'] ) . '" />';
			$string .= "\n</div>";
		} else {
			$string .= $captchaRequiresError;
		}
		return $string;
	}	// end function display_captcha()
	
	static function display_thank_you() {
		// Displays thank you message upon successful form submission
	
		// what gets printed after the form is sent, unless redirect is on.
		$ctf_form_style = FSCF_Display::convert_css(self::$form_options['form_style']);

		$ctf_thank_you = '
<!-- Fast Secure Contact Form plugin '.esc_html(FSCF_VERSION).' - begin - FastSecureContactForm.com -->
<div id="FSContact' . self::$form_id_num . '" ' . $ctf_form_style.'>
';

		if (self::$form_options['border_enable'] == 'true') {
			$ctf_thank_you .= '
    <fieldset '. FSCF_Display::convert_css(self::$form_options['border_style']) . '>
';
		if (self::$form_options['title_border'] != '')
			$ctf_thank_you .= '      <legend>'.esc_html(self::$form_options['title_border']).'</legend>';
		}


		$ctf_thank_you .= '
	<div '.FSCF_Display::convert_css(self::$form_options['redirect_style']).'>
';
		$ctf_thank_you .= (self::$form_options['text_message_sent'] != '') ? self::$form_options['text_message_sent'] : __('Your message has been sent, thank you.', 'si-contact-form'); // can have HTML

		if (FSCF_Process::$redirect_enable == 'true') {
			$ctf_thank_you .= '
		<br />
		<img src="'.plugins_url( 'ctf-loading.gif' , __FILE__ ).'" alt="'.esc_attr(__('Redirecting', 'si-contact-form')).'" />'.
		__('Redirecting', 'si-contact-form');
		}
		$ctf_thank_you .= '
	</div>';

		if (self::$form_options['border_enable'] == 'true') {
			$ctf_thank_you .= '
	</fieldset>';
		}
		$ctf_thank_you .= '
</div>
<!-- Fast Secure Contact Form plugin '.esc_html(FSCF_VERSION).' - end - FastSecureContactForm.com -->
';
	return($ctf_thank_you);
	}
	
	static function set_form_error($fld, $msg) {
		// Sets a form error for field $fld with message $msg
		// This is called from FSCF_Process class functions
		// The key is 'field' plus the field index number, or a special name such as
		// capctha, f_name, fscf_select, etc.
		self::$form_errors[$fld] = $msg;
		self::$contact_error = 1;	// Set the error flag
	}
	
	static function check_form_errors() {
		// Returns 1 if there are form errors, and 0 if not
		// Called from FSCF_Process::process_form()
		return(self::$contact_error);
	}

	static function ctf_notes($notes) {
		//return "\n    <div " . self::$style['notes']. ">\n$notes\n    </div>";
        return "\n    $notes\n"; // some users asked for the div to be removed
	}

	static function get_form_action_url() {
	// returns the URL for the WP page the form was on

		if ( function_exists( 'qtrans_convertURL' ) )
		// compatible with qtranslate plugin
		// In case of multi-lingual pages, the /de/ /en/ language url is used.
			$form_action_url = qtrans_convertURL( strip_tags( $_SERVER['REQUEST_URI'] ) );
		else
			$form_action_url =
					'http://' . strip_tags( $_SERVER['HTTP_HOST'] ) . strip_tags( $_SERVER['REQUEST_URI'] );

		// set the type of request (SSL or not)
		if ( is_ssl() )
			$form_action_url = preg_replace( '|http://|', 'https://', $form_action_url );

        //filter hook for form action URL
		return apply_filters( 'si_contact_form_action_url', $form_action_url,  self::$form_id_num);

	}  // end function form_action_url	
	
}  // end class FSCF_Display

?>